@@ -1,9 +1,6 @@
 //! On-chain types
 
 pub mod address;
-pub mod base16_bytes;
-pub mod block_id;
-pub mod digest32;
 pub mod ergo_box;
 pub mod header;
 #[cfg(feature = "json")]
