@@ -4,6 +4,7 @@ use crate::base16_str::Base16Str;
 use crate::bigint256::BigInt256;
 use crate::chain::ergo_box::ErgoBox;
 use crate::mir::value::CollKind;
+use crate::serialization::SigmaParsingError;
 use crate::serialization::SigmaSerializable;
 use crate::serialization::SigmaSerializationError;
 use crate::sigma_protocol::sigma_boolean::SigmaBoolean;
@@ -14,6 +15,8 @@ use crate::types::stuple::STuple;
 use crate::types::stuple::TupleItems;
 use crate::types::stype::LiftIntoSType;
 use crate::types::stype::SType;
+use ergo_chain_types::ADDigest;
+use ergo_chain_types::Base16DecodedBytes;
 use impl_trait_for_tuples::impl_for_tuples;
 use std::convert::TryFrom;
 use std::convert::TryInto;
@@ -24,6 +27,7 @@ mod constant_placeholder;
 pub use constant_placeholder::*;
 
 use super::avl_tree_data::AvlTreeData;
+use super::avl_tree_data::AvlTreeFlags;
 use super::value::NativeColl;
 use super::value::StoreWrapped;
 use super::value::Value;
@@ -393,6 +397,24 @@ impl From<AvlTreeData> for Constant {
     }
 }
 
+impl From<AvlTreeFlags> for Constant {
+    fn from(a: AvlTreeFlags) -> Self {
+        Constant {
+            tpe: SType::SByte,
+            v: Literal::Byte(a.serialize() as i8),
+        }
+    }
+}
+
+impl From<ADDigest> for Constant {
+    fn from(a: ADDigest) -> Self {
+        Constant {
+            tpe: SType::SColl(Box::new(SType::SByte)),
+            v: Literal::Coll(CollKind::NativeColl(NativeColl::CollByte(a.into()))),
+        }
+    }
+}
+
 #[allow(clippy::unwrap_used)]
 #[allow(clippy::from_over_into)]
 #[impl_for_tuples(2, 4)]
@@ -579,7 +601,7 @@ impl TryExtractFrom<Literal> for Vec<i8> {
 
 impl TryExtractFrom<Literal> for Vec<u8> {
     fn try_extract_from(v: Literal) -> Result<Self, TryExtractFromError> {
-        use crate::util::FromVecI8;
+        use sigma_util::FromVecI8;
         Vec::<i8>::try_extract_from(v).map(Vec::<u8>::from_vec_i8)
     }
 }
@@ -686,6 +708,14 @@ impl Base16Str for Constant {
     }
 }
 
+impl TryFrom<Base16DecodedBytes> for Constant {
+    type Error = SigmaParsingError;
+
+    fn try_from(value: Base16DecodedBytes) -> Result<Self, Self::Error> {
+        Constant::sigma_parse_bytes(&value.0)
+    }
+}
+
 #[cfg(feature = "arbitrary")]
 #[allow(clippy::unwrap_used)]
 #[allow(clippy::todo)]
@@ -754,14 +784,23 @@ pub(crate) mod arbitrary {
             SType::SGroupElement => any::<EcPoint>().prop_map_into().boxed(),
             SType::SSigmaProp => any::<SigmaProp>().prop_map_into().boxed(),
             SType::SBox => any::<ErgoBox>().prop_map_into().boxed(),
-            // SType::SAvlTree => {}
+            SType::SAvlTree => any::<AvlTreeData>().prop_map_into().boxed(),
             // SType::SOption(tpe) =>
+            SType::SOption(tpe) => match *tpe {
+                SType::SBoolean => any::<Option<bool>>().prop_map_into().boxed(),
+                SType::SByte => any::<Option<i8>>().prop_map_into().boxed(),
+                SType::SShort => any::<Option<i16>>().prop_map_into().boxed(),
+                SType::SInt => any::<Option<i32>>().prop_map_into().boxed(),
+                SType::SLong => any::<Option<i64>>().prop_map_into().boxed(),
+                _ => todo!(),
+            },
             SType::SColl(elem_tpe) => match *elem_tpe {
                 SType::SBoolean => vec(any::<bool>(), 0..400).prop_map_into().boxed(),
                 SType::SByte => vec(any::<u8>(), 0..400).prop_map_into().boxed(),
                 SType::SShort => vec(any::<i16>(), 0..400).prop_map_into().boxed(),
                 SType::SInt => vec(any::<i32>(), 0..400).prop_map_into().boxed(),
                 SType::SLong => vec(any::<i64>(), 0..400).prop_map_into().boxed(),
+                SType::SSigmaProp => vec(any::<SigmaProp>(), 0..3).prop_map_into().boxed(),
                 _ => todo!(),
             },
             // SType::STuple(_) => {}
