@@ -30,6 +30,7 @@ use super::constant::ConstantPlaceholder;
 use super::constant::Literal;
 use super::constant::TryExtractFrom;
 use super::constant::TryExtractFromError;
+use super::create_avl_tree::CreateAvlTree;
 use super::create_provedlog::CreateProveDlog;
 use super::decode_point::DecodePoint;
 use super::exponentiate::Exponentiate;
@@ -58,6 +59,7 @@ use super::sigma_and::SigmaAnd;
 use super::sigma_or::SigmaOr;
 use super::sigma_prop_bytes::SigmaPropBytes;
 use super::subst_const::SubstConstants;
+use super::tree_lookup::TreeLookup;
 use super::tuple::Tuple;
 use super::upcast::Upcast;
 use super::val_def::ValDef;
@@ -215,6 +217,10 @@ pub enum Expr {
     Exponentiate(Exponentiate),
     /// XOR for collection of booleans
     XorOf(XorOf),
+    /// Perform a lookup by key in an AVL tree
+    TreeLookup(TreeLookup),
+    /// Create an AVL tree
+    CreateAvlTree(CreateAvlTree),
 }
 
 impl Expr {
@@ -285,6 +291,8 @@ impl Expr {
             Expr::XorOf(v) => v.tpe(),
             Expr::ExtractBytes(v) => v.tpe(),
             Expr::ExtractBytesWithNoRef(v) => v.tpe(),
+            Expr::TreeLookup(v) => v.tpe(),
+            Expr::CreateAvlTree(v) => v.tpe(),
         }
     }
 
@@ -456,7 +464,11 @@ pub(crate) mod arbitrary {
             ]
             .prop_map_into()
             .boxed(),
-            SType::SSigmaProp => sigma_prop_nested_expr(depth),
+            SType::SSigmaProp => vec(sigma_prop_nested_expr(depth), 0..10)
+                .prop_map(|items| Collection::new(SType::SSigmaProp, items).unwrap())
+                .prop_map_into()
+                .boxed(),
+
             _ => panic!("Nested expression not implemented for {:?}", &elem_tpe),
         }
     }
