@@ -31,6 +31,7 @@ use crate::mir::collection::coll_sigma_parse;
 use crate::mir::collection::coll_sigma_serialize;
 use crate::mir::constant::Constant;
 use crate::mir::constant::ConstantPlaceholder;
+use crate::mir::create_avl_tree::CreateAvlTree;
 use crate::mir::create_prove_dh_tuple::CreateProveDhTuple;
 use crate::mir::create_provedlog::CreateProveDlog;
 use crate::mir::decode_point::DecodePoint;
@@ -65,6 +66,7 @@ use crate::mir::sigma_and::SigmaAnd;
 use crate::mir::sigma_or::SigmaOr;
 use crate::mir::sigma_prop_bytes::SigmaPropBytes;
 use crate::mir::subst_const::SubstConstants;
+use crate::mir::tree_lookup::TreeLookup;
 use crate::mir::tuple::Tuple;
 use crate::mir::upcast::Upcast;
 use crate::mir::val_def::ValDef;
@@ -139,6 +141,7 @@ impl Expr {
                 OpCode::DIVISION => Ok(bin_op_sigma_parse(ArithOp::Divide.into(), r)?),
                 OpCode::MAX => Ok(bin_op_sigma_parse(ArithOp::Max.into(), r)?),
                 OpCode::MIN => Ok(bin_op_sigma_parse(ArithOp::Min.into(), r)?),
+                OpCode::MODULO => Ok(bin_op_sigma_parse(ArithOp::Modulo.into(), r)?),
                 OpCode::BIT_OR => Ok(bin_op_sigma_parse(BitOp::BitOr.into(), r)?),
                 OpCode::BIT_AND => Ok(bin_op_sigma_parse(BitOp::BitAnd.into(), r)?),
                 OpCode::BIT_XOR => Ok(bin_op_sigma_parse(BitOp::BitXor.into(), r)?),
@@ -185,6 +188,8 @@ impl Expr {
                 MultiplyGroup::OP_CODE => Ok(MultiplyGroup::sigma_parse(r)?.into()),
                 Exponentiate::OP_CODE => Ok(Exponentiate::sigma_parse(r)?.into()),
                 XorOf::OP_CODE => Ok(XorOf::sigma_parse(r)?.into()),
+                TreeLookup::OP_CODE => Ok(TreeLookup::sigma_parse(r)?.into()),
+                CreateAvlTree::OP_CODE => Ok(CreateAvlTree::sigma_parse(r)?.into()),
                 o => Err(SigmaParsingError::NotImplementedOpCode(format!(
                     "{0}(shift {1})",
                     o.value(),
@@ -285,6 +290,8 @@ impl SigmaSerializable for Expr {
             Expr::XorOf(op) => op.sigma_serialize_w_opcode(w),
             Expr::ExtractBytes(op) => op.sigma_serialize_w_opcode(w),
             Expr::ExtractBytesWithNoRef(op) => op.sigma_serialize_w_opcode(w),
+            Expr::TreeLookup(op) => op.sigma_serialize_w_opcode(w),
+            Expr::CreateAvlTree(op) => op.sigma_serialize_w_opcode(w),
         }
     }
 
