@@ -16,6 +16,7 @@ use crate::sigma_protocol::sigma_boolean::{ProveDhTuple, ProveDlog};
 use super::sfunc::SFunc;
 use super::stuple::STuple;
 use super::stype_param::STypeVar;
+use crate::mir::avl_tree_data::AvlTreeData;
 
 /// Every type descriptor is a tree represented by nodes in SType hierarchy.
 #[derive(PartialEq, Eq, Debug, Clone)]
@@ -141,6 +142,12 @@ impl LiftIntoSType for bool {
     }
 }
 
+impl LiftIntoSType for u8 {
+    fn stype() -> SType {
+        SType::SByte
+    }
+}
+
 impl LiftIntoSType for i8 {
     fn stype() -> SType {
         SType::SByte
@@ -213,6 +220,12 @@ impl LiftIntoSType for ProveDhTuple {
     }
 }
 
+impl LiftIntoSType for AvlTreeData {
+    fn stype() -> SType {
+        SType::SAvlTree
+    }
+}
+
 impl<T: LiftIntoSType> LiftIntoSType for Option<T> {
     fn stype() -> SType {
         SType::SOption(Box::new(T::stype()))
